import * as Backbone from 'backbone'
import {props} from './decorators'
import {PostFormView, PostListView, PostView, CommentsView} from './views'


@props({
  routes: {
    '': 'index',
    'posts/new': 'newPost',
    'posts/:id': 'showPost',
    'posts/:id/comments': 'showComments'
  }
})
class AppRouter extends Backbone.Router {
  constructor({posts, $main}) {
    super();
    this.posts = posts;
    this.$main = $main;
  }
  index() {
    this.posts.fetch().then(() => {
      let v = new PostListView({collection: this.posts, router: this});
      this.$main.html(v.render().el);
    });
  }
  newPost() {
    let v = new PostFormView({
      posts: this.posts,
      router: this
    });
    this.$main.html(v.render().el);
  }
  showPost(id) {
    var v = new PostView({
      model: this.posts.get(id),
      router: this
    });
    this.$main.html(v.render().el);
  }
  showComments(postId) {
    var post = this.posts.get(postId);
    var v = new CommentsView({
      post: post,
      router: this
    });
    this.$main.html(v.render().el);
  }
}

export {
  AppRouter
}
