import * as Backbone from 'backbone'
import {tagName, template, on} from './decorators'
import postFormTemplate from '../templates/post-form.html'
import postListTemplate from '../templates/post-list.html'
import postListItemTemplate from '../templates/post-list-item.html'
import postTemplate from '../templates/post.html'
import commentFormTemplate from '../templates/comment-form.html'
import commentTemplate from '../templates/comment.html'
import commentsTemplate from '../templates/comments.html'

@tagName('form')
@template(postFormTemplate)
class PostFormView extends Backbone.View {
  constructor({posts, router}) {
    super();
    this.posts = posts;
    this.router = router;
  }
  render() {
    this.$el.html(this.template());
    return this;
  }
  @on('click .submit')
  submit(event) {
    event.preventDefault();
    this.posts.create({
      title: $('.title').val(),
      content: $('.content').val(),
      pubDate: new Date()
    }, {
      success: () => {
        this.router.navigate('/', {trigger: true});
      }
    });
  }
  @on('click .back')
  back(event) {
    event.preventDefault();
    this.router.navigate('/', {trigger: true});
  }
}

@tagName('li')
@template(postListItemTemplate)
class PostListItemView extends Backbone.View {
  render() {
    this.$el.html(this.template(this.model.toJSON()));
    return this;
  }
}

@template(postListTemplate)
class PostListView extends Backbone.View {
  constructor({collection, router}) {
    super({collection});
    this.router = router;
  }
  render() {
    this.$el.html(this.template());
    let $ul = this.$el.find('ul');
    this.collection.forEach(post => {
      $ul.append(new PostListItemView({model: post}).render().el);
    });
    return this;
  }
  @on('click a[href^="/posts/"]')
  handleClick(event) {
    event.preventDefault();
    let href = $(event.currentTarget).attr('href');
    this.router.navigate(href, {trigger: true});
  }
}

@template(postTemplate)
class PostView extends Backbone.View {
  constructor({model, posts, router}) {
    super({model});
    this.posts = posts;
    this.router = router;
  }
  render() {
    var model = this.model.toJSON();
    model.pubDate = new Date(model.pubDate).toDateString();
    this.$el.html(this.template(model));
    return this;
  }
  @on('click .comments')
  showComments(event) {
    this._handleClick(event);
  }
  @on('click .back')
  back(event) {
    this._handleClick(event);
  }
  _handleClick(event) {
    event.preventDefault();
    var href = $(event.currentTarget).attr('href');
    this.router.navigate(href, {trigger: true});
  }
}

@tagName('form')
@template(commentFormTemplate)
class CommentFormView extends Backbone.View {
  constructor({post, router}) {
    super();
    this.post = post;
    this.router = router;
  }
  render() {
    this.$el.html(this.template(this.post));
    return this;
  }
  @on('click .submit')
  submit(event) {
    event.preventDefault();
    this.post.comments.create({
      postId: this.post.get('id'),
      name: $('.name').val(),
      comment: $('.comment').val(),
      pubDate: new Date()
    });
    this.el.reset();
  }
  @on('click .back')
  back(event) {
    event.preventDefault();
    var href = $(event.currentTarget).attr('href');
    this.router.navigate(href, {trigger: true});
  }
}

@template(commentTemplate)
class CommentView extends Backbone.View {
  render() {
    let model = this.model.toJSON();
    model.pubDate = new Date(model.pubDate).toDateString();
    this.$el.html(this.template(model));
    return this;
  }
}

@template(commentsTemplate)
class CommentsView extends Backbone.View {
  constructor({post, router}) {
    super();
    this.post = post;
    this.router = router;
    this.post.comments.on('add', this.addComment.bind(this));
  }
  render() {
    this.$el.html(this.template());
    var v = new CommentFormView({
      post: this.post,
      router: this.router
    });
    this.$el.append(v.render().el);
    this.post.comments.fetch();
    return this;
  }
  addComment(comment) {
    var v = new CommentView({
      model: comment
    });
    this.$el.append(v.render().el);
  }
}


export {
  PostListView,
  PostFormView,
  PostView,
  CommentsView
}
